@@ -22,20 +22,21 @@ import Data.Char
 import Data.List
 
 import Control.Monad       ( liftM, filterM, when, guard )
-import Control.Monad.Error ( catchError, throwError, liftIO )
+import Control.Monad.Trans ( liftIO )
+import Control.Monad.Catch
 
 import Hint.Base
 import Hint.Util ( (>=>) ) -- compat version
 import Hint.Conversions
 import qualified Hint.Util   as Util
 import qualified Hint.Compat as Compat
+import qualified Hint.CompatPlatform as Compat
 
 import qualified Hint.GHC as GHC
 
 import System.Random
 import System.FilePath
 import System.Directory
-import qualified System.IO.UTF8 as UTF8 (writeFile)
 
 type ModuleText = String
 
@@ -44,12 +45,15 @@ type ModuleText = String
 -- already in-scope. Additionally, since this may be used with sandboxing in
 -- mind we want to avoid easy-to-guess names. Thus, we do a trick similar
 -- to the one in safeBndFor, but including a random number instead of an
--- additional digit
+-- additional digit. Finally, to avoid clashes between two processes
+-- that are concurrently running with the same random seed (e.g., initialized
+-- with the system time with not enough resolution), we also include the process id
 newPhantomModule :: MonadInterpreter m => m PhantomModule
 newPhantomModule =
     do n <- liftIO randomIO
+       p <- liftIO Compat.getPID
        (ls,is) <- allModulesInContext
-       let nums = concat [show (abs n::Int), filter isDigit $ concat (ls ++ is)]
+       let nums = concat [show (abs n::Int), show p, filter isDigit $ concat (ls ++ is)]
        let mod_name = 'M':nums
        --
        tmp_dir <- liftIO getTemporaryDirectory
@@ -67,7 +71,7 @@ addPhantomModule mod_text =
        let t  = Compat.fileTarget (pm_file pm)
            m  = GHC.mkModuleName (pm_name pm)
        --
-       liftIO $ UTF8.writeFile (pm_file pm) (mod_text $ pm_name pm)
+       liftIO $ writeFile (pm_file pm) (mod_text $ pm_name pm)
        --
        onState (\s -> s{active_phantoms = pm:active_phantoms s})
        mayFail (do -- GHC.load will remove all the modules from scope, so first
@@ -81,10 +85,10 @@ addPhantomModule mod_text =
                      then do runGhc2 Compat.setContext old_top old_imps
                              return $ Just ()
                      else return Nothing)
-        `catchError` (\err -> case err of
-                                WontCompile _ -> do removePhantomModule pm
-                                                    throwError err
-                                _             -> throwError err)
+        `catchIE` (\err -> case err of
+                             WontCompile _ -> do removePhantomModule pm
+                                                 throwM err
+                             _             -> throwM err)
        --
        return pm
 
@@ -107,7 +111,7 @@ removePhantomModule pm =
                      let mods' = filter (mod /=) mods
                      runGhc2 Compat.setContext mods' imps
                      --
-                     let isNotPhantom = isPhantomModule . fromGhcRep_  >=>
+                     let isNotPhantom = isPhantomModule . moduleToString  >=>
                                           return . not
                      null `liftM` filterM isNotPhantom mods'
              else return True
@@ -140,10 +144,29 @@ isPhantomModule mn = do (as,zs) <- getPhantomModules
 --
 -- The interpreter is 'reset' both before loading the modules and in the event
 -- of an error.
+--
+-- /IMPORTANT/: Like in a ghci session, this will also load (and interpret)
+--  any dependency that is not available via an installed package. Make
+--  sure that you are not loading any module that is also being used to
+--  compile your application.  In particular, you need to avoid modules
+--  that define types that will later occur in an expression that you will
+--  want to interpret.
+--
+-- The problem in doing this is that those types will have two incompatible
+-- representations at runtime: 1) the one in the compiled code and 2) the
+-- one in the interpreted code. When interpreting such an expression (bringing
+-- it to program-code) you will likely get a segmentation fault, since the
+-- latter representation will be used where the program assumes the former.
+--
+-- The rule of thumb is: never make the interpreter run on the directory
+-- with the source code of your program! If you want your interpreted code to
+-- use some type that is defined in your program, then put the defining module
+-- on a library and make your program depend on that package.
 loadModules :: MonadInterpreter m => [String] -> m ()
 loadModules fs = do -- first, unload everything, and do some clean-up
                     reset
-                    doLoad fs `catchError` (\e -> reset >> throwError e)
+                    doLoad fs `catchIE` (\e  -> reset >> throwM e)
+
 
 doLoad :: MonadInterpreter m => [String] -> m ()
 doLoad fs = do mayFail $ do
@@ -166,7 +189,7 @@ getLoadedModules = do (active_pms, zombie_pms) <- getPhantomModules
                       return $ ms \\ (map pm_name $ active_pms ++ zombie_pms)
 
 modNameFromSummary :: GHC.ModSummary -> ModuleName
-modNameFromSummary =  fromGhcRep_ . GHC.ms_mod
+modNameFromSummary =  moduleToString . GHC.ms_mod
 
 getLoadedModSummaries :: MonadInterpreter m => m [GHC.ModSummary]
 getLoadedModSummaries =
@@ -183,8 +206,8 @@ setTopLevelModules ms =
        --
        let not_loaded = ms \\ map modNameFromSummary loaded_mods_ghc
        when (not . null $ not_loaded) $
-         throwError $ NotAllowed ("These modules have not been loaded:\n" ++
-                                  unlines not_loaded)
+         throwM $ NotAllowed ("These modules have not been loaded:\n" ++
+                              unlines not_loaded)
        --
        active_pms <- fromState active_phantoms
        ms_mods <- mapM findModule (nub $ ms ++ map pm_name active_pms)
@@ -192,8 +215,8 @@ setTopLevelModules ms =
        let mod_is_interpr = runGhc1 GHC.moduleIsInterpreted
        not_interpreted <- filterM (liftM not . mod_is_interpr) ms_mods
        when (not . null $ not_interpreted) $
-         throwError $ NotAllowed ("These modules are not interpreted:\n" ++
-                                  unlines (map fromGhcRep_ not_interpreted))
+         throwM $ NotAllowed ("These modules are not interpreted:\n" ++
+                              unlines (map moduleToString not_interpreted))
        --
        (_, old_imports) <- runGhc Compat.getContext
        runGhc2 Compat.setContext ms_mods old_imports
@@ -203,7 +226,7 @@ onAnEmptyContext action =
     do (old_mods, old_imps) <- runGhc Compat.getContext
        runGhc2 Compat.setContext [] []
        let restore = runGhc2 Compat.setContext old_mods old_imps
-       a <- action `catchError` (\e -> do restore; throwError e)
+       a <- action `catchIE` (\e -> do restore; throwM e)
        restore
        return a
 
@@ -220,7 +243,7 @@ setImports ms = setImportsQ $ zip ms (repeat Nothing)
 -- | Sets the modules whose exports must be in context; some
 --   of them may be qualified. E.g.:
 --
---   @setImports [("Prelude", Nothing), ("Data.Map", Just "M")]@.
+--   @setImportsQ [("Prelude", Nothing), ("Data.Map", Just "M")]@.
 --
 --   Here, "map" will refer to Prelude.map and "M.map" to Data.Map.map.
 setImportsQ :: MonadInterpreter m => [(ModuleName, Maybe String)] -> m ()
@@ -302,15 +325,16 @@ installSupportModule = do mod <- addPhantomModule support_module
                                "    " ++ _show   ++ ")",
                                "where",
                                "",
-                               "import qualified Prelude as P",
+                               "import qualified Prelude as " ++ _P ++ " (String, Show(show))",
                                "",
-                               "type " ++ _String ++ " = P.String",
+                               "type " ++ _String ++ " = " ++ _P ++ ".String",
                                "",
-                               _show ++ " :: P.Show a => a -> P.String",
-                               _show ++ " = P.show"
+                               _show ++ " :: " ++ _P ++ ".Show a => a -> " ++ _P ++ ".String",
+                               _show ++ " = " ++ _P ++ ".show"
                              ]
             where _String = altStringName m
                   _show   = altShowName m
+                  _P      = altPreludeName m
 
 -- Call it when the support module is an active phantom module but has been
 -- unloaded as a side effect by GHC (e.g. by calling GHC.loadTargets)
@@ -325,6 +349,9 @@ altStringName mod_name = "String_" ++ mod_name
 altShowName :: ModuleName -> String
 altShowName mod_name = "show_" ++ mod_name
 
+altPreludeName :: ModuleName -> String
+altPreludeName mod_name = "Prelude_" ++ mod_name
+
 support_String :: MonadInterpreter m => m String
 support_String = do mod_name <- fromState (pm_name . hint_support_module)
                     return $ concat [mod_name, ".", altStringName mod_name]
