@@ -22,6 +22,9 @@ module Hint.GHC (
 #if __GLASGOW_HASKELL__ >= 702
     module SrcLoc,
 #endif
+#if __GLASGOW_HASKELL__ >= 708
+    module ConLike,
+#endif
 )
 
 where
@@ -35,11 +38,11 @@ import Bag ( bagToList )
 import GHC hiding ( Phase )
 #endif
 
-import Outputable   ( PprStyle, SDoc, ppr,
+import Outputable   ( PprStyle, SDoc, Outputable(ppr),
                       showSDoc, showSDocForUser, showSDocUnqual,
                       withPprStyle, defaultErrStyle )
 
-import ErrUtils     ( mkLocMessage  )
+import ErrUtils     ( mkLocMessage, pprErrMsgBagWithLoc)
 #if __GLASGOW_HASKELL__ < 706
 import ErrUtils     ( Message )
 #else
@@ -53,7 +56,9 @@ import Lexer        ( P(..), ParseResult(..), mkPState )
 import Parser       ( parseStmt, parseType )
 import FastString   ( fsLit )
 
-#if __GLASGOW_HASKELL__ >= 700
+#if   __GLASGOW_HASKELL__ >= 710
+import DynFlags     ( supportedLanguagesAndExtensions, xFlags, xopt, FlagSpec(..) )
+#elif __GLASGOW_HASKELL__ >= 700
 import DynFlags     ( supportedLanguagesAndExtensions, xFlags, xopt )
 #else
 import DynFlags     ( supportedLanguages )
@@ -63,10 +68,6 @@ import DynFlags     ( supportedLanguages )
 import DynFlags     ( LogAction )
 #endif
 
-#if __GLASGOW_HASKELL__ >= 706
-import DynFlags     ( tracingDynFlags )
-#endif
-
 #if __GLASGOW_HASKELL__ >= 608
 import PprTyThing   ( pprTypeForUser )
 #elif __GLASGOW_HASKELL__ < 608
@@ -77,6 +78,9 @@ import SrcLoc       ( SrcSpan )
 import SrcLoc       ( mkRealSrcLoc )
 #endif
 
+#if __GLASGOW_HASKELL__ >= 708
+import ConLike      ( ConLike(RealDataCon) )
+#endif
 
 #if __GLASGOW_HASKELL__ >= 706
 type Message = MsgDoc
