@@ -18,7 +18,15 @@ partition prop xs = foldr (select prop) ([],[]) xs
     where select p x ~(ts,fs) | p x       = (x:ts,fs)
                               | otherwise = (ts, x:fs)
 
+partitionEither :: [Either a b] -> ([a],[b])
+partitionEither [] = ([],[])
+partitionEither (Left  a:xs) = let (ls,rs) = partitionEither xs in (a:ls,rs)
+partitionEither (Right b:xs) = let (ls,rs) = partitionEither xs in (ls,b:rs)
+
+
 infixr 1 >=>
 (>=>) :: Monad m => (a -> m b) -> (b -> m c) -> (a -> m c)
 f >=> g = \x -> f x >>= g
 
+quote :: String -> String
+quote s = concat ["'", s, "'"]
